/*
 * @(#)Customer
 *
 * Copyright (c) 1998 Karl Moss. All Rights Reserved.
 *
 * You may study, use, modify, and distribute this software for any
 * purpose provided that this copyright notice appears in all copies.
 *
 * This software is provided WITHOUT WARRANTY either expressed or
 * implied.
 *
 * @author  Karl Moss
 * @version 1.0
 * @date    15May98
 *
 */

package javaservlets.rmi.server;
 
/**
 * <p>The server implementation for CustomerInterface. This
 * object will be instantiated remotely via RMI.
 */

public class Customer
  extends java.rmi.server.UnicastRemoteObject
  implements CustomerInterface
{
  // Database connection
  java.sql.Connection m_con;

  // Prepared Statement
  java.sql.PreparedStatement m_ps;
  
  // JDBC driver to register
  static String m_driver = "sun.jdbc.odbc.JdbcOdbcDriver";

  // Connection URL
  static String m_url = "jdbc:odbc:MyAccessDataSource";
  
  /**
    * <p>Default constructor.
    *
    * @param con JDBC Connection to use for the query
    */
  public Customer(java.sql.Connection con)
    throws java.rmi.RemoteException, java.sql.SQLException
    {
      super();
      m_con = con;

      // Create a prepared statement for us to use
      String sql = "SELECT Custno, Name, Balance from Customer " +
        "WHERE Custno = ?";

      m_ps = con.prepareStatement(sql);
    }
  
  /**
    * <p>Retrieves the customer data for the given customer ID
    * number. If the customer is not found a null value will be
    * returned.
    *
    * @param id Customer ID number
    * @return CustomerData object containing all of the customer
    * data or null if not found
    */
  public CustomerData getCustomerData(String id)
    throws java.rmi.RemoteException, java.sql.SQLException
    {
      CustomerData data = null;
    
      System.out.println("Customer query for " + id);
      
      // Set the customer ID
      m_ps.setInt(1, Integer.parseInt(id));
      
      // Execute the query
      java.sql.ResultSet rs = m_ps.executeQuery();

      // Get the results. If there are no results available,
      // return null to the client
      if (rs.next()) {

        // A row exists. Create a new CustomerData object and
        // fill it in
        data = new CustomerData();
        data.id = rs.getString(1);
        data.name = rs.getString(2);
        data.balance = rs.getBigDecimal(3, 2);
      }

      // Close the ResultSet
      rs.close();
      
      return data;
    }

  /**
    * <p>Main entry point for the remote object. This method
    * will bootstrap the object and register it with the
    * RMI registry.
    */
  public static void main(String args[])
    {
      // Install the default RMI security manager
      System.setSecurityManager(new java.rmi.RMISecurityManager());
      
      try {

        // Register the JDBC driver
        Class.forName(m_driver).newInstance();

        System.out.println("Opening database connection");
        
        // Create a new JDBC connection
        java.sql.Connection con =
          java.sql.DriverManager.getConnection(m_url);
        
        // Create a new instance of the server object
        Customer cust = new Customer(con);

        // Bind the object to the RMI registry. If the object is
        // already bound it will be replaced
        java.rmi.Naming.rebind("/Customer", cust);

        System.out.println("Customer server object ready.");
      }
      catch (Exception ex) {

        // Display any errors
        ex.printStackTrace();
      }
    }
}
